#!/bin/sh
#
# Perto S.A.
#
# blacklist-usblp.sh
#
# Script to remove the usblp module to avoid the libusb/usblp conflict for the Perto Printer TEC.
#

echo ""
echo "Remove usblp module to avoid the libusb/usblp conflict for the Perto Printer TEC"

# Add usblp module to the blacklist to avoid the libusb/usblp conflict permanently
if [ -f /etc/modprobe.d/50-blacklist.conf ]
then
    # In openSuse the file name is 50-blacklist.conf
    grep "blacklist usblp" /etc/modprobe.d/50-blacklist.conf > /dev/null
    if [ $? -ne 0 ]
    then
        # If usblp is not in the blacklist then add it permanently
        echo "Inserting 'blacklist usblp' in /etc/modprobe.d/50-blacklist.conf to avoid the libusb/usblp conflict permanently..."
        sudo sh -c 'printf "\n# replaced by libusb (Perto Printer TEC)\nblacklist usblp\n" >> /etc/modprobe.d/50-blacklist.conf'
    fi
else
    # In Ubuntu the file name is blacklist.conf
    grep "blacklist usblp" /etc/modprobe.d/blacklist.conf > /dev/null
    if [ $? -ne 0 ]
    then
        # If usblp is not in the blacklist then add it permanently
        echo "Inserting 'blacklist usblp' in /etc/modprobe.d/blacklist.conf to avoid the libusb/usblp conflict permanently..."
        sudo sh -c 'printf "\n# replaced by libusb (Perto Printer TEC)\nblacklist usblp\n" >> /etc/modprobe.d/blacklist.conf'
    fi
fi

# Remove usblp module to avoid the libusb/usblp conflict immediately
lsmod | grep usblp
if [ $? -eq 0 ]
then
    # If usblp is running then remove it and restart cups
    echo "Remove usblp and restart cups to avoid the libusb/usblp conflict now..."
    sudo rmmod -v usblp 2> /dev/null
    if [ $? -eq 0 ]
    then
        sudo /etc/init.d/cups restart
    fi
fi

echo "usblp removed"
