#!/bin/bash
#
# Perto S.A.
#
# install-ptptpro-linux.sh
#
# Script to install the lib ptptrpro for Perto Printer TEC from the ptptrpro-linux.tar.gz contents.
#

# Abort the shell script if any command returns a non-zero value
set -e

# Definitions
LIB_PATH="ptptrpro-linux"
PREFIX="/usr"
CURR_DIR=$PWD

ARCH="$(uname -m)"

# Install ptptrpro
function Installptptrpro()
{
	# Install libs
	echo ""
	echo "Installing the lib ptptrpto and Qt libs..."
	sudo install -d $PREFIX/lib/

	sudo install -s -m 644 $LIB_PATH/lib/libptptrpro-linux.so.1.0.0 $PREFIX/lib/
	sudo ln -sf libptptrpro-linux.so.1.0.0 $PREFIX/lib/libptptrpro-linux.so
	sudo ln -sf libptptrpro-linux.so.1.0.0 $PREFIX/lib/libptptrpro-linux.so.1
	sudo ln -sf libptptrpro-linux.so.1.0.0 $PREFIX/lib/libptptrpro-linux.so.1.0

	sudo install -s -m 644 $LIB_PATH/lib/libicudata.so.49.1.2 $PREFIX/lib/
	sudo ln -sf libicudata.so.49.1.2 $PREFIX/lib/libicudata.so.49

	sudo install -s -m 644 $LIB_PATH/lib/libicui18n.so.49.1.2 $PREFIX/lib/
	sudo ln -sf libicui18n.so.49.1.2 $PREFIX/lib/libicui18n.so.49

	sudo install -s -m 644 $LIB_PATH/lib/libicuuc.so.49.1.2 $PREFIX/lib/
	sudo ln -sf libicuuc.so.49.1.2 $PREFIX/lib/libicuuc.so.49

	sudo install -s -m 644 $LIB_PATH/lib/libQt5Core.so.5.0.2 $PREFIX/lib/
	sudo ln -sf libQt5Core.so.5.0.2 $PREFIX/lib/libQt5Core.so
	sudo ln -sf libQt5Core.so.5.0.2 $PREFIX/lib/libQt5Core.so.5
	sudo ln -sf libQt5Core.so.5.0.2 $PREFIX/lib/libQt5Core.so.5.0

	sudo install -s -m 644 $LIB_PATH/lib/libQt5Gui.so.5.0.2 $PREFIX/lib/
	sudo ln -sf libQt5Gui.so.5.0.2 $PREFIX/lib/libQt5Gui.so
	sudo ln -sf libQt5Gui.so.5.0.2 $PREFIX/lib/libQt5Gui.so.5
	sudo ln -sf libQt5Gui.so.5.0.2 $PREFIX/lib/libQt5Gui.so.5.0
	
	
	echo ""
	echo "Lib ptptrpro installation is done"
	echo ""
}
# Install libs
function InstallUSB() 
{
	
	echo "Install the lib ptptrpro components to build applications for the Perto Printer TEC"
	echo ""
	read -p "Turn ON and connect the cable USB the printer before continue. Press [Enter] to continue..." key
	echo ""

	# Install udev rules to change usb device permission for Perto Printer TEC
	echo ""
	echo "Installing the udev rules for libusb..."

	sudo install -m 644 ptptrpro-linux/udev-usb-libusb.rules /etc/udev/rules.d/

	echo ""
	echo "Installing the libusb 1.0..."
	cd $CURR_DIR/libusb-1.0.9
	./configure --prefix=$CURR_DIR/libusb-linux-bin-1.0.9
	make
	make install
	echo ""
	echo "Installing the libusb 0.1 compatibility layer..."
	echo ""
	echo "NOTICE: The libusb 0.1 installation path cannot be too long"
	echo ""
	cd $CURR_DIR/libusb-compat-0.1.5
	echo $PKG_CONFIG_PATH
	export PKG_CONFIG_PATH="$CURR_DIR/libusb-linux-bin-1.0.9/lib/pkgconfig"
	./configure --prefix=$CURR_DIR/libusb-linux-bin-1.0.9
	make
	make install
	cd $CURR_DIR

	# Remove the usblp module to avoid the libusb/usblp conflict 
	./blacklist-usblp.sh

	echo ""
	read -p "Turn OFF the printer before continue. Press [Enter] to continue..." key
	read -p "Turn ON and connect the printer before continue. Press [Enter] to continue..." key

	echo ""
	echo "Libusb driver installation is done"
	echo ""
}



echo ""
echo "Perto Printer TEC script installer"
echo ""
echo "Options"
PS3='select communication port: '
options=("USB (libusb)" "Serial port")
select opt in "${options[@]}"
do
    case $opt in
        "USB (libusb)")	

			if [[ $ARCH == "x86_64" ]]
  			then
	   			echo "Architecture "$ARCH" not supported on this installation option"
				exit 1 		
  			fi

			Installptptrpro		
            InstallUSB            
			break;
            ;;
        "Serial port")
            Installptptrpro  
			break;                
            ;;
        *)  echo ""
			echo "Invalid option!"
			echo ""
			exit 1
			;;
    esac
done
