#!/bin/sh
#
# Perto S.A.
#
# install-cups.sh
#
# Script to install the Perto Printer TEC CUPS driver from the pertotec-cups.tar.gz contents.
#

# Abort the shell script if any command returns a non-zero value
set -e

# Definitions
CUPS_PATH="pertotec-cups"
PREFIX="/usr"
CURR_DIR=$PWD

echo ""
echo "Install the libusb and the Perto Printer TEC CUPS driver"
echo ""
read -p "Turn ON and connect the printer before continue. Press [Enter] to continue..." key
echo ""

echo "Installing the libusb 1.0..."
cd $CURR_DIR/libusb-1.0.9
./configure --prefix=$CURR_DIR/libusb-linux-bin-1.0.9
make
make install
echo ""
echo "Installing the libusb 0.1 compatibility layer..."
echo ""
echo "NOTICE: The libusb 0.1 installation path cannot be too long"
echo ""
cd $CURR_DIR/libusb-compat-0.1.5
echo $PKG_CONFIG_PATH
export PKG_CONFIG_PATH="$CURR_DIR/libusb-linux-bin-1.0.9/lib/pkgconfig"
./configure --prefix=$CURR_DIR/libusb-linux-bin-1.0.9
make
make install
cd $CURR_DIR

# Install CUPS driver
echo "Installing the CUPS driver..."
sudo install -d $PREFIX/lib/cups/filter/
sudo install -d $PREFIX/lib/cups/backend/
sudo install -d $PREFIX/share/cups/model/
sudo install -s -m 755 $CUPS_PATH/rastertoperto $PREFIX/lib/cups/filter/
sudo install -s -m 755 $CUPS_PATH/pertousb $PREFIX/lib/cups/backend/
sudo install -m 644 $CUPS_PATH/pertotec.ppd $PREFIX/share/cups/model

# Install udev rules to change usb device permission for Perto Printer TEC
echo ""
echo "Installing the udev rules for libusb..."
sudo install -m 644 $CUPS_PATH/udev-usb-libusb.rules /etc/udev/rules.d/

# Remove the usblp module to avoid the libusb/usblp conflict 
./blacklist-usblp.sh

echo ""
read -p "Turn OFF the printer before continue. Press [Enter] to continue..." key
read -p "Turn ON and connect the printer before continue. Press [Enter] to continue..." key
echo ""
echo "CUPS driver installation is done"
echo ""
