/**
 * 	Copyright (c) 2016 Perto S.A. All rights reserved.
 *
 * \file main.cpp
 *
 * \brief Test
 *
 * \author Abilio Faria
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ptptrpro.h"

/*! Define INCLUDE_UNSUPPORTED to test unsupported API functions */
// #define INCLUDE_UNSUPPORTED

/*!
 * Macros
 */

/* macro to handle return code */
#define LOG_RET(func_name,ret) {                                \
        switch (ret) {                                          \
        case PTR_OK:                                            \
            printf("%40s: Passed\n", func_name);                \
            break;                                              \
        case PTR_ERR_UNSUPPORTED:                               \
            printf("%40s: Not supported.\n", func_name);        \
            break;                                              \
        default:                                                \
            printf("%40s: Failed! Error=%d\n", func_name, ret); \
            return;                                             \
        }                                                       \
    }

//! Clean keyboard buffer
#define FLUSH \
    do {\
        char unused[2]; \
        fgets(unused, sizeof(unused), stdin); \
    } while(0)

/*!
 * Prototypes
 */
void TestPrinter(char *port, int baudrate);
void TestStatus(void);
void TestPaperCut(void);
void TestPrint(void);
void TestBarCode(void);
void TestBitImage(void);
void TestCharacter(void);
void TestLayout(void);
void TestMechanismControl(void);
void TestCancel(void);
void TestPrintMode(void);
void TestMacro(void);
void TestLog(void);
int SendText(const char *text);
int PrintText(const char *text);
void TestValidator(void);
void TestQrCode(void);
void TestNFe(void);
void TestClosePort(void);
void TestGraphics(void);
void TestSaveGraphic(void);
void TestCleanAllGraphics(void);
void TestCleanGraphic(void);
void TestPrintGraphic(void);

/*!
 * Private variables
 */

/* raster bit image "ESC/POS", x=16, y=40 */
static unsigned char raster_bit_image[640] =
{0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
 0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,
 0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,
 0xCF,0xFF,0xE0,0xFF,0xE0,0x3F,0xF8,0x00,0x5F,0xFF,0x80,0x1F,0xFC,0x00,0xFF,0xE3,
 0xCF,0xFF,0xE3,0xFF,0xE0,0xFF,0xF8,0x00,0x5F,0xFF,0xE0,0x7F,0xFF,0x03,0xFF,0xE3,
 0xCF,0xFF,0xE7,0xFF,0xE1,0xFF,0xF8,0x00,0xDF,0xFF,0xE0,0xFF,0xFF,0x87,0xFF,0xE3,
 0xCF,0xFF,0xE7,0xFF,0xE1,0xFF,0xF8,0x00,0x9F,0xFF,0xF0,0xFF,0xFF,0x87,0xFF,0xE3,
 0xCF,0xFF,0xEF,0xFF,0xE3,0xFF,0xF8,0x00,0x9F,0xFF,0xF1,0xFF,0xFF,0xCF,0xFF,0xE3,
 0xCF,0xFF,0xEF,0xFF,0xE3,0xFF,0xF8,0x01,0x9F,0xFF,0xF1,0xFF,0xFF,0xCF,0xFF,0xE3,
 0xCF,0x80,0x0F,0xE0,0x07,0xF0,0x00,0x01,0x1F,0x83,0xF9,0xF8,0x0F,0xCF,0xE0,0x03,
 0xCF,0x80,0x0F,0xC0,0x07,0xF0,0x00,0x01,0x1F,0x81,0xF9,0xF8,0x0F,0xCF,0xC0,0x03,
 0xCF,0x80,0x0F,0xC0,0x07,0xE0,0x00,0x03,0x1F,0x81,0xFB,0xF0,0x07,0xEF,0xC0,0x03,
 0xCF,0x80,0x0F,0xC0,0x07,0xE0,0x00,0x02,0x1F,0x81,0xFB,0xF0,0x07,0xEF,0xC0,0x03,
 0xCF,0x80,0x0F,0xE0,0x07,0xE0,0x00,0x06,0x1F,0x81,0xFB,0xF0,0x07,0xEF,0xE0,0x03,
 0xCF,0x80,0x0F,0xF0,0x07,0xE0,0x00,0x04,0x1F,0x81,0xFB,0xF0,0x07,0xEF,0xF0,0x03,
 0xCF,0xFF,0xC7,0xF8,0x07,0xE0,0x00,0x04,0x1F,0x81,0xFB,0xF0,0x07,0xE7,0xF8,0x03,
 0xCF,0xFF,0xC7,0xFC,0x07,0xE0,0x00,0x0C,0x1F,0x81,0xFB,0xF0,0x07,0xE7,0xFC,0x03,
 0xCF,0xFF,0xC3,0xFE,0x07,0xE0,0x00,0x08,0x1F,0x81,0xFB,0xF0,0x07,0xE3,0xFE,0x03,
 0xCF,0xFF,0xC1,0xFF,0x07,0xE0,0x00,0x18,0x1F,0x83,0xFB,0xF0,0x07,0xE1,0xFF,0x03,
 0xCF,0xFF,0xC0,0xFF,0x87,0xE0,0x00,0x18,0x1F,0xFF,0xF3,0xF0,0x07,0xE0,0xFF,0x83,
 0xCF,0xFF,0xC0,0x7F,0xC7,0xE0,0x00,0x10,0x1F,0xFF,0xF3,0xF0,0x07,0xE0,0x7F,0xC3,
 0xCF,0x80,0x00,0x3F,0xE7,0xE0,0x00,0x30,0x1F,0xFF,0xF3,0xF0,0x07,0xE0,0x3F,0xE3,
 0xCF,0x80,0x00,0x1F,0xE7,0xE0,0x00,0x20,0x1F,0xFF,0xE3,0xF0,0x07,0xE0,0x1F,0xE3,
 0xCF,0x80,0x00,0x0F,0xF7,0xE0,0x00,0x20,0x1F,0xFF,0xE3,0xF0,0x07,0xE0,0x0F,0xF3,
 0xCF,0x80,0x00,0x07,0xF7,0xE0,0x00,0x60,0x1F,0xFF,0x83,0xF0,0x07,0xE0,0x07,0xF3,
 0xCF,0x80,0x00,0x03,0xF7,0xE0,0x00,0x40,0x1F,0x80,0x03,0xF0,0x07,0xE0,0x03,0xF3,
 0xCF,0x80,0x00,0x03,0xF7,0xE0,0x00,0xC0,0x1F,0x80,0x03,0xF0,0x07,0xE0,0x03,0xF3,
 0xCF,0x80,0x00,0x03,0xF7,0xF0,0x00,0x80,0x1F,0x80,0x01,0xF8,0x0F,0xC0,0x03,0xF3,
 0xCF,0x80,0x00,0x07,0xF7,0xF0,0x00,0x80,0x1F,0x80,0x01,0xF8,0x0F,0xC0,0x07,0xF3,
 0xCF,0xFF,0xEF,0xFF,0xF3,0xFF,0xF9,0x80,0x1F,0x80,0x01,0xFF,0xFF,0xCF,0xFF,0xF3,
 0xCF,0xFF,0xEF,0xFF,0xF3,0xFF,0xF9,0x00,0x1F,0x80,0x01,0xFF,0xFF,0xCF,0xFF,0xF3,
 0xCF,0xFF,0xEF,0xFF,0xF3,0xFF,0xF9,0x00,0x1F,0x80,0x00,0xFF,0xFF,0x8F,0xFF,0xF3,
 0xCF,0xFF,0xEF,0xFF,0xE1,0xFF,0xFB,0x00,0x1F,0x80,0x00,0xFF,0xFF,0x8F,0xFF,0xF3,
 0xCF,0xFF,0xEF,0xFF,0xC0,0xFF,0xFA,0x00,0x1F,0x80,0x00,0x7F,0xFF,0x0F,0xFF,0xC3,
 0xCF,0xFF,0xEF,0xFF,0x00,0x3F,0xFA,0x00,0x1F,0x80,0x00,0x1F,0xFC,0x0F,0xFF,0x03,
 0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,
 0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,
 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};


/*!< usage message */
static const char *usage_message =
    "- ptptrpro-test: Aplicativo de testes da impressora termica\n"
    "          com o uso da API PTPTRPRO.\n\n"
    "- Para utilizar pela USB: ptptpro-test.exe ( sem parametros )\n"
    "- Para utilizar pela USBLP: ptptrpro-test.exe -s <portaUSBLP>"
    "- Para utilizar pela Serial: ptptrpro-test.exe -s <portaCOM> <baudrate>\n\n"
    "- Exemplo:\n"
    "  ptptrpro-test.exe\n  ptptrpro-test.exe -s /dev/usb/lp0 (somente no linux)\n  ptptrpro-test.exe -s COM1 (/dev/ttyS0 no Linux) 115200 \n\n"
;

/*!
 * \brief main
 */
int main(int argc, char *argv[])
{
    char port[16];
    int baudrate = 0;

    memset(port, 0x00, sizeof(port));
    switch(argc)
    {
         case 1:
             strcpy(port, "USB");
             break;
         case 3:
            if(strcmp(argv[1], "-s") == 0)
            {
                strcpy(port, argv[2]);
            }
            break;
         case 4:
            if(strcmp(argv[1], "-s") == 0)
            {
                strcpy(port, argv[2]);
                baudrate = atoi(argv[3]);
            }
         break;

         default:
            printf("%s", usage_message);
            return EXIT_FAILURE;

     }


    TestPrinter(port, baudrate);

}



/*!
 * \brief Test the printer
 */
void TestPrinter(char *port, int baudrate)
{
    int ret;

    printf("PTPTRPRO Printer Test\n\n");

    ret = SetProtocol(PTR_PROT_ESCPOS);
    LOG_RET("SetProtocol", ret);

    if(strcmp(port, "USB") == 0)
    {
        ret = OpenPort(port, 0, '\0', 0, 0);
    }
    else
    {
        ret = OpenPort(port, baudrate, 'n', 8, 1);
    }

    LOG_RET("OpenPort", ret);

    ret = Reset();
    LOG_RET("Reset", ret);


    TestStatus();
    TestPrint();
    TestBarCode();
    TestBitImage();
    TestCharacter();
    TestLayout();
    TestMechanismControl();
    TestCancel();
    TestPrintMode();
    TestMacro();
    TestLog();
    TestQrCode();

    //TestPaperCut();
    MakeCut(PTR_CUT_PARTIAL);

    TestNFe();

    TestGraphics();

    if (SetOperationMode(PTR_MODE_VALIDATION) == PTR_OK){
        TestValidator();
    }

   // system("pause");
    TestClosePort();

    printf("\nPTPTRPRO Printer Passed!\n");
}

/*!
 * \brief Test status functions
 */
void TestStatus(void)
{
    int ret;
    int status;
    PrinterInfoType info;

#ifdef INCLUDE_UNSUPPORTED
    ret = RealTimeRequest(1);
    LOG_RET("RealTimeRequest not supported", ret);
#endif

    ret = GetPrinterStatus( &status);
    LOG_RET("GetPrinterStatus", ret);
    printf("Printer status=%02xH\n", status);

    ret = ptptrpro_GetMediaStatus( &status);
    LOG_RET("GetMediaStatus", ret);
    printf("Media status=%02xH\n", status);

    // TBD: how to test drawer open/close signal (connector pin 3)?
    ret = GetDrawerStatus(PTR_DRAWERONE, &status);
    LOG_RET("GetDrawerStatus", ret);
    printf("Drawer status=0x%02x\n", status);

    ret = GetPrinterInfo(&info);
    LOG_RET("GetPrinterInfo", ret);
    printf("Printer model=%d\n", info.printer_model);
    printf("Printer type=%d\n", info.type);
    printf("Printer version=%s\n", info.rom_version);

    ret = GetRealTimeStatus(1, &status, 0);
    LOG_RET("GetRealTimeStatus", ret);
    printf("Printer status 1=%02xH\n", status);
    ret = GetRealTimeStatus(2, &status, 0);
    LOG_RET("GetRealTimeStatus", ret);
    printf("Printer status 2=%02xH\n", status);
    ret = GetRealTimeStatus(3, &status, 0);
    LOG_RET("GetRealTimeStatus", ret);
    printf("Printer status 3=%02xH\n", status);
    ret = GetRealTimeStatus(4, &status, 0);
    LOG_RET("GetRealTimeStatus", ret);
    printf("Printer status 4=%02xH\n", status);
}

/*!
 * \brief Test paper cut functions
 */
void TestPaperCut(void)
{
    int ret;

    PrintText("MakeCut partial");
    ret = MakeCut(PTR_CUT_PARTIAL);
    LOG_RET("MakeCut", ret);
    // TBD: the cut was not partial. It was full.

#ifdef INCLUDE_UNSUPPORTED
    PrintText("MakeCut full not supported");
    ret = MakeCut(PTR_CUT_FULL);    /* not supported */
    LOG_RET("MakeCut not supported", ret);
#endif

    PrintText("MakeCutnLines 1, 0");
    ret = MakeCutnLines(1, 0);
    LOG_RET("MakeCutnLines", ret);
    // TBD: the cut was not partial. It was full.

    PrintText("MakeCutnLines 66, 30");
    ret = MakeCutnLines(66, 30);
    LOG_RET("MakeCutnLines", ret);
    // TBD: the cut was not partial. It was full.
}

/*!
 * \brief Test print functions
 */
void TestPrint(void)
{
    int ret;
    int len;

    len = SendText("PrintAndLineFeed");
    ret = PrintAndLineFeed(len);
    LOG_RET("PrintAndLineFeed", ret);

    len = SendText("PrintAndPaperFeed (20 dots)");
    ret = PrintAndPaperFeed(20);
    LOG_RET("PrintAndPaperFeed", ret);

#ifdef INCLUDE_UNSUPPORTED
    len = SendText("PrintInPageMode not supported ");
    ret = PrintInPageMode();
    LOG_RET("PrintInPageMode not supported", ret);
#endif

    len = SendText("MakePrint");
    ret = MakePrint();
    LOG_RET("MakePrint", ret);

    len = SendText("GetPartialLineFeed (3 lines)");
    ret = GetPartialLineFeed(3);
    LOG_RET("GetPartialLineFeed", ret);

    len = SendText("PaperFeed (255 units)");
    ret = PaperFeed(255);
    LOG_RET("PaperFeed", ret);

    len = SendText("SetDumpEnable");
    ret = SetDumpEnable();
    LOG_RET("SetDumpEnable", ret);
}

/*!
 * \brief Test bar code functions
 */
void TestBarCode(void)
{
    int ret = 0;

    ret = SetHriFont(PTR_BAR_CODE_FONT_NORMAL);
    LOG_RET("SetHriFont", ret);
    ret = SetHriFont(PTR_BAR_CODE_FONT_NARROW);
    LOG_RET("SetHriFont", ret);

    ret = SetHriPosition(PTR_BAR_CODE_POS_NO);
    LOG_RET("SetHriPosition", ret);
    ret = SetHriPosition(PTR_BAR_CODE_POS_ABOVE);
    LOG_RET("SetHriPosition", ret);
    ret = SetHriPosition(PTR_BAR_CODE_POS_BELOW);
    LOG_RET("SetHriPosition", ret);
    ret = SetHriPosition(PTR_BAR_CODE_POS_ABOVEANDBELOW);
    LOG_RET("SetHriPosition", ret);

    PrintText("PrintBarCode UPCA");
    ret = PrintBarCode(PTR_BAR_CODE_UPCA,               /* type */
                       50,                              /* height */
                       PTR_BAR_CODE_WIDTH_MEDIUM,       /* width */
                       0,                               /* left_margin */
                       PTR_BAR_CODE_POS_NO,             /* text_pos */
                       PTR_BAR_CODE_FONT_NORMAL,        /* text_font */
                       "12345678901");                  /* data */
    // result: works only for size 11
    LOG_RET("PrintBarCode UPCA", ret);

    PrintText("PrintBarCode UPCE");
    ret = PrintBarCode(PTR_BAR_CODE_UPCE,               /* type */
                       50,                              /* height */
                       PTR_BAR_CODE_WIDTH_MEDIUM,       /* width */
                       0,                               /* left_margin */
                       PTR_BAR_CODE_POS_BELOW,          /* text_pos */
                       PTR_BAR_CODE_FONT_NORMAL,        /* text_font */
                       "123456");                  /* data */
    // result: failed. printing only the text (only size 11), not the bar code.
    LOG_RET("PrintBarCode UPCE", ret);

    PrintText("PrintBarCode EAN13");
    ret = PrintBarCode(PTR_BAR_CODE_EAN13,              /* type */
                       50,                              /* height */
                       PTR_BAR_CODE_WIDTH_MEDIUM,       /* width */
                       0,                               /* left_margin */
                       PTR_BAR_CODE_POS_BELOW,          /* text_pos */
                       PTR_BAR_CODE_FONT_NORMAL,        /* text_font */
                       "123456789012");                 /* data */
    // result: works only for size 12
    LOG_RET("PrintBarCode EAN13", ret);

    PrintText("PrintBarCode EAN8");
    ret = PrintBarCode(PTR_BAR_CODE_EAN8,               /* type */
                       50,                              /* height */
                       PTR_BAR_CODE_WIDTH_MEDIUM,       /* width */
                       0,                               /* left_margin */
                       PTR_BAR_CODE_POS_ABOVEANDBELOW,  /* text_pos */
                       PTR_BAR_CODE_FONT_NORMAL,        /* text_font */
                       "1234567");                      /* data */
    // result: works only for size 7
    LOG_RET("PrintBarCode EAN8", ret);

    PrintText("PrintBarCode CODE39");
    ret = PrintBarCode(PTR_BAR_CODE_CODE39,             /* type */
                       50,                              /* height */
                       PTR_BAR_CODE_WIDTH_MEDIUM,       /* width */
                       0,                               /* left_margin */
                       PTR_BAR_CODE_POS_BELOW,          /* text_pos */
                       PTR_BAR_CODE_FONT_NORMAL,        /* text_font */
                       "123456789");            /* data */
    // result: failed
    LOG_RET("PrintBarCode CODE39", ret);

    PrintText("PrintBarCode ITF");
    ret = PrintBarCode(PTR_BAR_CODE_ITF,                /* type */
                       50,                              /* height */
                       PTR_BAR_CODE_WIDTH_MEDIUM,       /* width */
                       0,                               /* left_margin */
                       PTR_BAR_CODE_POS_BELOW,          /* text_pos */
                       PTR_BAR_CODE_FONT_NORMAL,        /* text_font */
                       "12345678901234567890");         /* data */
    // result: passed
    LOG_RET("PrintBarCode ITF", ret);

    PrintText("PrintBarCode CODABAR");
    ret = PrintBarCode(PTR_BAR_CODE_CODABAR,            /* type */
                       50,                              /* height */
                       PTR_BAR_CODE_WIDTH_MEDIUM,       /* width */
                       0,                               /* left_margin */
                       PTR_BAR_CODE_POS_ABOVEANDBELOW,  /* text_pos */
                       PTR_BAR_CODE_FONT_NORMAL,        /* text_font */
                       "ABCD-abcd-1234567890");         /* data */
    // result: the first and last character were missed and lower case was converted to upper case.
    LOG_RET("PrintBarCode CODABAR", ret);

    PrintText("PrintBarCode CODE93");
    ret = PrintBarCode(PTR_BAR_CODE_CODE93,             /* type */
                       50,                              /* height */
                       PTR_BAR_CODE_WIDTH_MEDIUM,       /* width */
                       0,                               /* left_margin */
                       PTR_BAR_CODE_POS_BELOW,          /* text_pos */
                       PTR_BAR_CODE_FONT_NORMAL,        /* text_font */
                       "CODE93 1234567890");            /* data */
    // result: passed
    LOG_RET("PrintBarCode CODE93", ret);

    PrintText("PrintBarCode CODE128");
    ret = PrintBarCode(PTR_BAR_CODE_CODE128,            /* type */
                       50,                              /* height */
                       PTR_BAR_CODE_WIDTH_MEDIUM,       /* width */
                       0,                               /* left_margin */
                       PTR_BAR_CODE_POS_BELOW,          /* text_pos */
                       PTR_BAR_CODE_FONT_NORMAL,        /* text_font */
                       "CODE128 1234567890");           /* data */
    // result: failed
    LOG_RET("PrintBarCode CODE128", ret);

}

/*!
 * \brief Test bit image functions
 */
void TestBitImage(void)
{
    int ret;
    int i;
    unsigned char buffer[70];

    PrintText("PrintBmpImage perto-logo.bmp 2x");
    ret = PrintBmpImage("perto-logo.bmp",      /* char *file_name */
                        2,       /* height */
                        2,       /* width */
                        0,       /* degree */
                        0,       /* left_margin */
                        PTR_IMAGE_DITHER_2X2);      /* dither_mode */

    LOG_RET("PrintBmpImage perto-logo.bmp 2x", ret);

    PrintText("PrintBmp8 perto-logo.bmp");
    ret = PrintBmp8("perto-logo.bmp",      /* char *file_name */
                    1,       /* height */
                    1,       /* width */
                    0,       /* degree */
                    0,       /* left_margin */
                    PTR_IMAGE_DITHER_2X2);      /* dither_mode */
    LOG_RET("PrintBmp8 perto-logo.bmp", ret);

    PrintText("PrintRasterBitImage");
    ret = PrintRasterBitImage(0, 16*8, 40, raster_bit_image);     /* mode, width, height, data */
    LOG_RET("PrintRasterBitImage", ret);

#ifdef INCLUDE_UNSUPPORTED
    PrintText("PrintDownloadedBitImage not supported");
    ret = PrintDownloadedBitImage(0);               /* mode */
    LOG_RET("PrintDownloadedBitImage not supported", ret);

    PrintText("SetDownloadedBitImage not supported");
    ret = SetDownloadedBitImage(0, 1, buffer);       /* x, y, data */
    LOG_RET("SetDownloadedBitImage not supported", ret);
#endif

    /* init buffer for SetBitImageMode */
    i = 0;
    while (i < 70) {
        buffer[i++] = 1;    buffer[i++] = 2;    buffer[i++] = 4;    buffer[i++] = 8;
        buffer[i++] = 16;   buffer[i++] = 32;   buffer[i++] = 64;   buffer[i++] = 128;
        buffer[i++] = 64;   buffer[i++] = 32;   buffer[i++] = 16;   buffer[i++] = 8;
        buffer[i++] = 4;    buffer[i++] = 2;
    }

    PrintText("SetBitImageMode 1");
    ret = SetBitImageMode(1, buffer, 70);             /* mode, data, size */
    ret = SetBitImageMode(1, buffer, 70);             /* mode, data, size */
    SetLineSpacing(0);
    MakePrint();
    SetDefaultLineSpacing();
    ret = SetBitImageMode(1, buffer, 70);             /* mode, data, size */
    ret = SetBitImageMode(1, buffer, 70);             /* mode, data, size */
    MakePrint();    /* print image in the buffer */
    LOG_RET("SetBitImageMode 1", ret);
    // question: mode 1 works as single or double density?


    PrintText("SetBitImageMode 33");
    ret = SetBitImageMode(33, buffer, 69);             /* mode, data, size */
    ret = SetBitImageMode(33, buffer, 69);             /* mode, data, size */
    SetLineSpacing(0);
    MakePrint();
    SetDefaultLineSpacing();
    ret = SetBitImageMode(33, buffer, 69);             /* mode, data, size */
    ret = SetBitImageMode(33, buffer, 69);             /* mode, data, size */
    MakePrint();    /* print image in the buffer */
    LOG_RET("SetBitImageMode 33", ret);
    // question: mode 33 works as single or double density?

#ifdef INCLUDE_UNSUPPORTED
    PrintText("SetBitImageMode 0 not supported");
    ret = SetBitImageMode(0, buffer, 70);             /* mode, data, size */
    MakePrint();    /* print image in the buffer */
    LOG_RET("SetBitImageMode 0  not supported", ret);

    PrintText("SetBitImageMode 32  not supported");
    ret = SetBitImageMode(32, buffer, 69);             /* mode, data, size */
    MakePrint();    /* print image in the buffer */
    LOG_RET("SetBitImageMode 32  not supported", ret);
#endif
}

/*!
 * \brief Test character functions
 */
void TestCharacter(void)
{
    int ret;
#ifdef INCLUDE_UNSUPPORTED
    char data[1];
#endif

    /*
     *      bit 0: Character font: 0 (font A (12 x 24)), 1 (font B (9 x 17))
     *      bit 1: Undefined
     *      bit 2: Undefined
     *      bit 3: Emphasized mode: 0 (off), 1 (on)
     *      bit 4: Double-height mode: 0 (off), 1 (on)
     *      bit 5: Double-width mode: 0 (off), 1 (on)
     *      bit 6: Undefined
     *      bit 7: Underline mode: 0 (off), 1 (on)
     */
    ret = SetPrintMode(0xFF);
    LOG_RET("SetPrintMode FFH", ret);
    PrintText("SetPrintMode FFH");
    ret = SetPrintMode(0x01);
    LOG_RET("SetPrintMode 01H Font B", ret);
    PrintText("SetPrintMode 01H Font B");
    ret = SetPrintMode(0x00);
    LOG_RET("SetPrintMode 00H Font A", ret);
    PrintText("SetPrintMode 00H Font A");

    ret = SetCharCodeTable(PTR_CODE_TABLE_PC437);
    LOG_RET("SetCharCodeTable USA 9FH=\x9F", ret);
    PrintText("SetCharCodeTable USA 9FH=\x9F");
    ret = SetCharCodeTable(PTR_CODE_TABLE_PC860);
    LOG_RET("SetCharCodeTable Portuguese 9FH=\x9F", ret);
    PrintText("SetCharCodeTable Portuguese 9FH=\x9F");

    ret = SetTextProperties(PTR_FONT_NARROW, true, true, true);     /* font, italic, underlined, bold */
    LOG_RET("SetTextProperties NARROW", ret);
    PrintText("SetTextProperties NARROW");
    ret = SetTextProperties(PTR_FONT_EXPANDED, true, true, true);     /* font, italic, underlined, bold */
    LOG_RET("SetTextProperties EXPANDED", ret);
    PrintText("SetTextProperties EXPANDED");
    ret = SetTextProperties(PTR_FONT_NORMAL, true, false, false);     /* font, italic, underlined, bold */
    LOG_RET("SetTextProperties NORMAL italic", ret);
    // result: italic failed
    PrintText("SetTextProperties NORMAL italic");
    ret = SetTextProperties(PTR_FONT_NORMAL, false, false, false);     /* font, italic, underlined, bold */
    LOG_RET("SetTextProperties NORMAL", ret);
    PrintText("SetTextProperties NORMAL");

    ret = SetFontType(1);
    LOG_RET("SetFontType 1 font B", ret);
    PrintText("SetFontType 1 font B");
    ret = SetFontType(0);
    LOG_RET("SetFontType 0 font A", ret);
    PrintText("SetFontType 0 font A");

    ret = SetCharSize(PTR_CHAR_SIZE_DOUBLE);
    LOG_RET("SetCharSize 11H double", ret);
    PrintText("SetCharSize 11H double");
    ret = SetCharSize(PTR_CHAR_SIZE_DOUBLE_HEIGHT);
    LOG_RET("SetCharSize 01H double-height", ret);
    PrintText("SetCharSize 01H double-height");
    ret = SetCharSize(PTR_CHAR_SIZE_DOUBLE_WIDTH);
    LOG_RET("SetCharSize 10H double-width", ret);
    PrintText("SetCharSize 10H double-width");
    ret = SetCharSize(PTR_CHAR_SIZE_NORMAL);
    LOG_RET("SetCharSize 00H normal", ret);
    PrintText("SetCharSize 00H normal");
    // result: bits 1, 2, 5, 6 must be 0.

    ret = SetItalicEnable();
    LOG_RET("SetItalicEnable", ret);
    PrintText("SetItalicEnable");
    // result: italic failed.

    ret = SetItalicDisable();
    LOG_RET("SetItalicDisable", ret);
    PrintText("SetItalicDisable");

    ret = SetPrintModeUnderline(1);
    LOG_RET("SetPrintModeUnderline 1", ret);
    PrintText("SetPrintModeUnderline 1");
    ret = SetPrintModeUnderline(2);
    LOG_RET("SetPrintModeUnderline 2", ret);
    PrintText("SetPrintModeUnderline 2");
    ret = SetPrintModeUnderline(0);
    LOG_RET("SetPrintModeUnderline 0", ret);
    PrintText("SetPrintModeUnderline 0");

    ret = SetBoldEnable(0);
    LOG_RET("SetBoldEnable", ret);
    PrintText("SetBoldEnable");

    ret = SetBoldDisable();
    LOG_RET("SetBoldDisable", ret);
    PrintText("SetBoldDisable");

    ret = SetDoubleStrike(1);
    LOG_RET("SetDoubleStrike 1", ret);
    PrintText("SetDoubleStrike 1");
    ret = SetDoubleStrike(0);
    LOG_RET("SetDoubleStrike 0", ret);
    PrintText("SetDoubleStrike 0");

    ret = SetReverseBlackWhite(1);
    LOG_RET("SetReverseBlackWhite 1", ret);
    PrintText("SetReverseBlackWhite 1");
    ret = SetReverseBlackWhite(0);
    LOG_RET("SetReverseBlackWhite 0", ret);
    PrintText("SetReverseBlackWhite 0");

#ifdef INCLUDE_UNSUPPORTED
    ret = SetCondensedEnable();
    LOG_RET("SetCondensedEnable not supported", ret);
    PrintText("SetCondensedEnable not supported");

    ret = SetCondensedDisable();
    LOG_RET("SetCondensedDisable not supported", ret);
    PrintText("SetCondensedDisable not supported");

    ret = SetExpandedModeEnable();
    LOG_RET("SetExpandedModeEnable not supported", ret);
    PrintText("SetExpandedModeEnable not supported");

    ret = SetExpandedModeDisable();
    LOG_RET("SetExpandedModeDisable not supported", ret);
    PrintText("SetExpandedModeDisable not supported");

    ret = SetInternationalCharSet(0);
    LOG_RET("SetInternationalCharSet not supported", ret);
    PrintText("SetInternationalCharSet not supported");

    ret = SetUserChar(0);
    LOG_RET("SetUserChar not supported", ret);
    PrintText("SetUserChar not supported");

    ret = DefineUserChar(0, 0, 0, data);   /* height, start_code, final_code, data */
    LOG_RET("DefineUserChar not supported", ret);
    PrintText("DefineUserChar not supported");

    ret = CancelUserChar(0);
    LOG_RET("CancelUserChar not supported", ret);
    PrintText("CancelUserChar not supported");
#endif
}

/*!
 * \brief Test layout functions
 */
void TestLayout(void)
{
    int ret;
    unsigned char tabs[6] = {6, 12, 18, 24, 30, 0};

#ifdef INCLUDE_UNSUPPORTED
    ret = SetMotionUnits(1, 1);
    LOG_RET("SetMotionUnits not supported", ret);
    PrintText("SetMotionUnits not supported");
#endif

    ret = SetPrintingArea(0, 0, 100, 0);  /* x, y, width, height */
    LOG_RET("SetPrintingArea 0 0 100 0", ret);
    PrintText("SetPrintingArea 0 0 100 0");
    ret = SetPrintingArea(0, 0, 50, 0);  /* x, y, width, height */
    LOG_RET("SetPrintingArea 0 0 50 0", ret);
    PrintText("SetPrintingArea 0 0 50 0");
    // result: parm width is not stable, and area height does not seem to make sense (it is character height).

#ifdef INCLUDE_UNSUPPORTED
    ret = SetPrintingAreaHeight(0);
    LOG_RET("SetPrintingAreaHeight not supported", ret);
    PrintText("SetPrintingAreaHeight not supported");

    ret = SetPageLength(10);
    LOG_RET("SetPageLength not supported", ret);
    PrintText("SetPageLength not supported");
#endif

    ret = SetLeftMargin(20);
    LOG_RET("SetLeftMargin 40", ret);
    PrintText("SetLeftMargin 40...");
    PrintText("SetLeftMargin 40");
    // result: not stable
    ret = SetLeftMargin(0);
    LOG_RET("SetLeftMargin 0", ret);
    PrintText("SetLeftMargin 0...");
    PrintText("SetLeftMargin 0");

    ret = SetRightMargin(50);
    LOG_RET("SetRightMargin 50", ret);
    PrintText("SetRightMargin 50...");
    PrintText("SetRightMargin 50");
    ret = SetRightMargin(100);
    LOG_RET("SetRightMargin 100", ret);
    PrintText("SetRightMargin 100...");
    PrintText("SetRightMargin 100");

    ret = SetJustification(2);
    LOG_RET("SetJustification right", ret);
    PrintText("SetJustification right");
    ret = SetJustification(1);
    LOG_RET("SetJustification center", ret);
    PrintText("SetJustification center");
    ret = SetJustification(0);
    LOG_RET("SetJustification left", ret);
    PrintText("SetJustification left");

#ifdef INCLUDE_UNSUPPORTED
    ret = SetCharSpacing(1);
    LOG_RET("SetCharSpacing not supported", ret);
    PrintText("SetCharSpacing not supported");
#endif

    ret = SetLineSpacing(30);
    LOG_RET("SetLineSpacing", ret);
    PrintText("SetLineSpacing ...");
    PrintText("SetLineSpacing 30");

    ret = SetDefaultLineSpacing();
    LOG_RET("SetDefaultLineSpacing", ret);
    PrintText("SetDefaultLineSpacing...");
    PrintText("SetDefaultLineSpacing");

    SendText("---");
    ret = SetRelativePrintPosition(30);
    LOG_RET("SetRelativePrintPosition 30", ret);
    PrintText("SetRelativePrintPosition 30");

    SendText("---");
    ret = SetAbsolutePrintPosition(30);
    LOG_RET("SetAbsolutePrintPosition 30", ret);
    PrintText("SetAbsolutePrintPosition 30");

#ifdef INCLUDE_UNSUPPORTED
    ret = SetAbsoluteVerticalPrintPosition(1);
    LOG_RET("SetAbsoluteVerticalPrintPosition not supported", ret);
    PrintText("SetAbsoluteVerticalPrintPosition not supported");
#endif

    PrintText("SetHorizTabPosition:");
    PrintText("1\t2\t3\t4\t");
    ret = SetHorizTabPosition(tabs);
    LOG_RET("SetHorizTabPosition", ret);
    PrintText("1\t2\t3\t4\t");

#ifdef INCLUDE_UNSUPPORTED
    ret = SetCharacterPerLine(0);
    LOG_RET("SetCharacterPerLine not supported", ret);
    PrintText("SetCharacterPerLine not supported");

    ret = SetAutomaticLineFeed(0);
    LOG_RET("SetAutomaticLineFeed not supported", ret);
    PrintText("SetAutomaticLineFeed not supported");
#endif

    ret = SetUpsideDownMode(1);
    LOG_RET("SetUpsideDownMode", ret);
    PrintText("SetUpsideDownMode on");

    ret = SetUpsideDownMode(0);
    LOG_RET("SetUpsideDownMode", ret);
    PrintText("SetUpsideDownMode off");

#ifdef INCLUDE_UNSUPPORTED
    ret = SetPrintDensity(2);
    LOG_RET("SetPrintDensity not supported", ret);
    PrintText("SetPrintDensity not supported");

    ret = SetMode48Columns();
    LOG_RET("SetMode48Columns not supported", ret);
    PrintText("SetMode48Columns not supported");

    ret = SetRotation(0);
    LOG_RET("SetRotation not supported", ret);
    PrintText("SetRotation not supported");
#endif
}

/*!
 * \brief Test mechanism control functions
 */
void TestMechanismControl(void)
{
    int ret;

    ret = GeneratePulse(0, 10, 10);     /* pin, time_on, time_off */
    LOG_RET("GeneratePulse 0 10 10", ret);
    PrintText("GeneratePulse 0 10 10");
    ret = GeneratePulse(1, 100, 100);     /* pin, time_on, time_off */
    LOG_RET("GeneratePulse 1 100 100", ret);
    PrintText("GeneratePulse 1 100 100");
    // result: how to test

#ifdef INCLUDE_UNSUPPORTED
    ret = SetPeripheralDevice(0);
    LOG_RET("SetPeripheralDevice not supported", ret);
    PrintText("SetPeripheralDevice not supported");

    ret = SetKeyboard(0);
    LOG_RET("SetKeyboard not supported", ret);
    PrintText("SetKeyboard not supported");

    ret = SetDrawerOn(0);
    LOG_RET("SetDrawerOn not supported", ret);
    PrintText("SetDrawerOn not supported");
#endif
}

/*!
 * \brief Test cancel functions
 */
void TestCancel(void)
{
#ifdef INCLUDE_UNSUPPORTED
    int ret;

    ret = CancelLastCharacterSent();
    LOG_RET("CancelLastCharacterSent not supported", ret);
    PrintText("CancelLastCharacterSent not supported");

    ret = CancelLastLineSended();
    LOG_RET("CancelLastLineSended not supported", ret);
    PrintText("CancelLastLineSended not supported");
#endif
}

/*!
 * \brief Test print mode functions
 */
void TestPrintMode(void)
{
    int ret;

    ret = SetPageMode();
    LOG_RET("SetPageMode", ret);
    PrintText("SetPageMode");

    ret = SetStandardMode();
    LOG_RET("SetStandardMode", ret);
    PrintText("SetStandardMode");

}

/*!
 * \brief Test macro functions
 */
void TestMacro(void)
{
#ifdef INCLUDE_UNSUPPORTED
    int ret;

    ret = StartEndMacroDefinition();
    LOG_RET("StartEndMacroDefinition not supported", ret);
    PrintText("StartEndMacroDefinition not supported");

    ret = ExecuteMacro(1, 1, 0);    /* times, waiting_time, mode */
    LOG_RET("ExecuteMacro not supported", ret);
    PrintText("ExecuteMacro not supported");
#endif
}

/*!
 * \brief Test log functions
 */
void TestLog(void)
{
#ifdef INCLUDE_UNSUPPORTED
    int ret;

    ret = EnableLog(0, "log.txt");      /* mode, file_name */
    LOG_RET("EnableLog not supported", ret);
    PrintText("EnableLog not supported");
#endif
}

/*!
 * \brief Send text to printer buffer
 */
int SendText(const char *text)
{
    char len;
    int ret;

    len = strlen(text);
    ret = SendData(text, len);
    if (ret != PTR_OK) {
        printf("%20s: Failed! Error=%d\n", "SendData", ret);
        len = 0;
    }
    return len;
}

/*!
 * \brief Send text to printer buffer and print it
 */
int PrintText(const char *text)
{
    int len;

    len = SendText(text);
    MakePrint();
    //GetPartialLineFeed(1);
    return len;
}

/*!
 * \brief Test validator functions
 */
void TestValidator(void)
{
    int ret;
    int status;


//   while(1){

//        status = 0;
//        ret = GetPrinterStatus( &status);
//        printf("GetPrinterStatus %X\n", status);

//        status = 0;
//        ret = ptptrpro_GetMediaStatus( &status);
//        printf("ptptrpro_ptptrpro_GetMediaStatus %X\n", status);

//    }


    printf("\n\nColoque o papel no validador (E nao remova ate finalizar a operaçao)\n");

    Reset();

    do
    {
        ret = ptptrpro_GetMediaStatus( &status);

    }while(status & PTR_MEDIA_ST_VALIDATION_NOPAPER);


    ret = SetTextProperties(PTR_FONT_NORMAL, false, false, false);     /* font, italic, underlined, bold */
    LOG_RET("SetTextProperties NORMAL", ret);
    PrintText("SetTextProperties NORMAL");


   printf("Retire o papel no validador\n");

   do
   {
       ret = ptptrpro_GetMediaStatus( &status);

   }while(!(status & PTR_MEDIA_ST_VALIDATION_NOPAPER));

    printf("Coloque o papel no validador (E nao remova ate finalizar a operaçao)\n");

   Reset();

   do
   {
       ret = ptptrpro_GetMediaStatus( &status);

   }while(status & PTR_MEDIA_ST_VALIDATION_NOPAPER);

   ret = SetTextProperties(PTR_FONT_NORMAL, true, false, false);     /* font, italic, underlined, bold */
   LOG_RET("SetTextProperties ITALIC", ret);
   PrintText("SetTextProperties ITALIC");


    printf("Retire o papel no validador\n");

    do
    {
        ret = ptptrpro_GetMediaStatus( &status);

    }while(!(status & PTR_MEDIA_ST_VALIDATION_NOPAPER));

    printf("Coloque o papel no validador (E nao remova ate finalizar a operaçao)\n");

    Reset();

    do
    {
        ret = ptptrpro_GetMediaStatus( &status);

    }while(status & PTR_MEDIA_ST_VALIDATION_NOPAPER);

    ret = SetTextProperties(PTR_FONT_NORMAL, false, false, true);     /* font, italic, underlined, bold */
    LOG_RET("SetTextProperties BOLD", ret);
    PrintText("SetTextProperties BOLD");


     printf("Retire o papel no validador\n");

     do
     {
         ret = ptptrpro_GetMediaStatus( &status);

     }while(!(status & PTR_MEDIA_ST_VALIDATION_NOPAPER));

     printf("Coloque o papel no validador (E nao remova ate finalizar a operaçao)\n");

    Reset();

    do
    {
        ret = ptptrpro_GetMediaStatus( &status);

    }while(status & PTR_MEDIA_ST_VALIDATION_NOPAPER);

    ret = SetTextProperties(PTR_FONT_EXPANDED, false, false, false);     /* font, italic, underlined, bold */
    LOG_RET("SetTextProperties EXPANDED", ret);
    PrintText("SetTextProperties EXPANDED");

    printf("Retire o papel no validador\n");

    do
    {
        ret = ptptrpro_GetMediaStatus( &status);

    }while(!(status & PTR_MEDIA_ST_VALIDATION_NOPAPER));

    printf("Coloque o papel no validador (E nao remova ate finalizar a operaçao)\n");

    Reset();

    do
    {
        ret = ptptrpro_GetMediaStatus( &status);

    }while(status & PTR_MEDIA_ST_VALIDATION_NOPAPER);

    ret = SetTextProperties(PTR_FONT_NARROW, false, false, false);     /* font, italic, underlined, bold */
    LOG_RET("SetTextProperties NARROW", ret);
    PrintText("SetTextProperties NARROW");

    printf("Retire o papel no validador\n");
}


void TestQrCode(void)
{
    int ret = 0;
    char data[100];
    memset(data, 0x00, sizeof(data));
    strcpy(data, "http://www.perto.com.br/pt/produtos/varejo/pertoprinter.html");

    ret = SetJustification(PTR_POS_CENTER);
    PrintText("QRCode examples");

    ret = SetJustification(PTR_POS_LEFT);
    PrintText("Resolution 3");

    ret = SetQrCodeModel( PTR_QRCODE_MODELTWO );
    LOG_RET("SetQrCodeModel", ret);

    ret = SetQrCodeResolution( PTR_QRCODE_FACTOR2 );
    LOG_RET("SetQrCodeResolution", ret);

    ret = SetQrCodeErrorCorretion(PTR_QRCODE_LEVEL_L );
    LOG_RET("SetQrCodeErrorCorretion", ret);

    ret = StoreQrCodeData(data, strlen(data));
    LOG_RET("StoreQrCodeData", ret);

    ret = SetJustification(PTR_POS_CENTER);
    LOG_RET("SetJustification", ret);

    ret = MakeQrCode();
    LOG_RET("MakeQrCode", ret);


    ///
    ret = SetJustification(PTR_POS_LEFT);
    PrintText("Resolution 5");

    ret = SetQrCodeModel( PTR_QRCODE_MODELTWO );
    LOG_RET("SetQrCodeModel", ret);

    ret = SetQrCodeResolution( PTR_QRCODE_FACTOR5 );
    LOG_RET("SetQrCodeResolution", ret);
    ret = SetQrCodeErrorCorretion(PTR_QRCODE_LEVEL_M );
    LOG_RET("SetQrCodeErrorCorretion", ret);

    ret = StoreQrCodeData(data, strlen(data));
    LOG_RET("StoreQrCodeData", ret);

    ret = SetJustification(PTR_POS_CENTER);
    LOG_RET("SetJustification", ret);

    ret = MakeQrCode();
    LOG_RET("MakeQrCode", ret);


    ///
    ret = SetJustification(PTR_POS_LEFT);
    PrintText("Resolution 7");
    ret = SetQrCodeModel( PTR_QRCODE_MODELTWO );
    LOG_RET("SetQrCodeModel", ret);

    ret = SetQrCodeResolution( PTR_QRCODE_FACTOR7 );
    LOG_RET("SetQrCodeResolution", ret);
    ret = SetQrCodeErrorCorretion(PTR_QRCODE_LEVEL_H );
    LOG_RET("SetQrCodeErrorCorretion", ret);

    ret = StoreQrCodeData(data, strlen(data));
    LOG_RET("StoreQrCodeData", ret);

    ret = SetJustification(PTR_POS_CENTER);
    LOG_RET("SetJustification", ret);

    ret = MakeQrCode();
    LOG_RET("MakeQrCode", ret);

    ret = SetJustification(PTR_POS_CENTER);
}

void TestNFe()
{
    int ret = 0;
    char data[500];
    memset(data, 0x00, sizeof(data));

    strcpy(data, "http://www.nfe.se.gov.br/portal/consultarNFCe.jsp?chNFe=43120910585504000174650010000000541123456781&nVersao=100&tpAmb=1&cDest=43708379006485&dhEmi=323031322d30392d32375431363a32303a33342d30333a3030&vNF=1000.00&vICMS=180.00&digVal=37327151612b623074616f514f3966414a7766646c5875715176383d&cIdToken=000001&cHashQRCode=80f5d4a1e4b12bc97aae0e971a61bff73270fd81");

    ret = SetJustification(PTR_POS_CENTER);
    PrintText("Exemplo de NFe");
    ret = PaperFeed(30);

    ret = SetFontType(PTR_BAR_CODE_FONT_NARROW);
    ret = SetCharCodeTable(PTR_CODE_TABLE_PC850);
    LOG_RET("SetCharCodeTable", ret);
    ret = SetLineSpacing(28);


    ret = SetJustification(PTR_POS_CENTER);
    PrintText("EMPRESA BRASILEIRA DE CORREIOS");
    PrintText("CNPJ 34.028.316/0001-03");
    PrintText("Inscrição Estadual - XXXXXXXXX");
    PrintText("NOME DA RUA, NUMERO NA RUA, BAIRRO, CIDADE UF");

    ret = SetJustification(PTR_POS_LEFT);
    PrintText("--------------------------------------------------------");

    ret = SetJustification(PTR_POS_CENTER);
    PrintText("DANFE NFC-e - Documento Auxiliar");
    PrintText("da Nota Fiscal de Consumidor Eletrónica");
    PrintText("Não permite aproveitamento de crédito de ICMS");

    ret = SetJustification(PTR_POS_LEFT);
    PrintText("--------------------------------------------------------");

    ret = SetJustification(PTR_POS_CENTER);
    PrintText("Detalhe da Venda");

    ret = SetJustification(PTR_POS_LEFT);
    PrintText("--------------------------------------------------------");
    PrintText("ITEM  CÓDIGO         DESCRIÇÃO                          ");
    PrintText(" QTD.   UN     VL. UNIT(R$)                VL. TOTAL(R$)");
    PrintText("--------------------------------------------------------");
    PrintText("001 0000000000001 EXEMPLO DE PRODUTO 1 - 100G       1,25");
    PrintText("002 0000000000002 EXEMPLO DE PRODUTO 2 - UNIDADE");
    PrintText(" 1.000 PC X 7,90                                    7,90");
    PrintText("003 0000000000003 EXEMPLO DE PRODUTO 3 - UNIDADE");
    PrintText(" 1.000 PC X 8,90                                    8,90");
    PrintText("004 0000000000004 EXEMPLO DE PRODUTO 4 - 2 UNIDADES");
    PrintText(" 2.000 LI X 2,27                                    4,54");
    PrintText("--------------------------------------------------------");
    PrintText("QTD. TOTAL DE ITENS                                    4");
    PrintText("VALOR TOTAL R$                                     22,59");
    PrintText("FORMA DE PAGAMENTO                            VALOR PAGO");
    PrintText("Cartão Débito                                      22,58");
    PrintText("--------------------------------------------------------");

    ret = SetJustification(PTR_POS_CENTER);
    PrintText("Número 111111 Série 111  Emissão 26/06/2015 14:26:10    ");
    PrintText("Via Consumidor");
    PrintText("Consulte pela Chave de Acesso em:");
    PrintText("https://www.sefaz.rs.gov.br/NFCE/NFCE-COM.aspx");
    PrintText("CHAVE DE ACESSO");
    PrintText("0000 1111 2222 3333 4444 5555 6666 7777 8888 9999 0000");
    PrintText("--------------------------------------------------------");
    PrintText("CONSUMIDOR");
    PrintText("CPF/CNPJ/ID Estrang: CONSUMIDOR NÃO IDENTIFICADO");
    PrintText("--------------------------------------------------------");

    PrintText("Consulta via leitor de QR Code");
    ret = SetQrCodeModel( PTR_QRCODE_MODELTWO );    //seleciona QR Code modelo 02
    LOG_RET("SetQrCodeModel", ret);

    ret = SetQrCodeResolution( 4 );     //seleciona QR Code fator 04
    LOG_RET("SetQrCodeResolution", ret);

    ret = SetQrCodeErrorCorretion(PTR_QRCODE_LEVEL_M );//seleciona QR Code correcao de erros M (0x31)
    LOG_RET("SetQrCodeErrorCorretion", ret);

    ret = StoreQrCodeData(data, strlen(data));
    LOG_RET("StoreQrCodeData", ret);

    ret = MakeQrCode();
    LOG_RET("MakeQrCode", ret);

    PrintText("Protocolo de Autorização: 143140099271631");
    PrintText("26/06/2015  14:26:10");


    PrintText("--------------------------------------------------------");
    PrintText("Val Aprox Tributos R$ 6,47 (28.64%) Fonte:IBPT");
    PrintText("         CIELO-VISA ELECTRON");
    PrintText("        999999******8888-08/28");
    PrintText("PDV=33333333 DOC=222222 AUT=111111");
    PrintText("VENDA A DEBITO");
    PrintText("VALOR:                 22,59   (SiTef)");

    MakeCut(PTR_CUT_PARTIAL);

}

void TestClosePort(void)
{
    int ret;
    ret = ClosePort();
    LOG_RET("ClosePort", ret);
}


void TestGraphics(void)
{

    printf("\n Graphic utility PertoPrinter TEC.\n");
    char op = 0;
    int ret = 0, mem = 0;

    do
    {       

        printf("\n\t Select the desired function: \n\n");
        printf("\t1 - Save graphic in printer\n");
        printf("\t2 - Print specific graphic\n");
        printf("\t3 - Get Memory Availabled\n");
        printf("\t4 - Remove specific graphic\n");
        printf("\t5 - Remove all graphics\n");
        printf("\n\tE - Exit\n");
        printf("\n\tOption: ");
        scanf("%c", &op);

        FLUSH;

        printf("\n");
        switch(op)
        {
            case '1':
            {
                TestSaveGraphic();
                break;
            }
            case '2':
            {
                TestPrintGraphic();
                break;
            }
            case '3':
            {
                ret = GetMemoryAvailabled(&mem);
                LOG_RET("GetMemoryAvailabled", ret);
                printf("\n\tMemory Availabled [ %d bytes ]\n", mem);
                break;
            }
            case '4':
            {
                TestCleanGraphic();
                break;
            }
            case '5':
            {
                TestCleanAllGraphics();
                break;
            }

        }

        if (op == 'E' || op =='e')
        {
             break;
        }
        else if (op != '1' && op != '2' && op != '3' && op != '4'&& op != '5')
        {
            op = 0;
            printf("Invalid option!\n");
            continue;
        }



    }while(1);

}



void TestSaveGraphic(void)
{
    char file_name[2048];
    memset(file_name, 0x00, sizeof(file_name));

    int ret = -1;
    int kc1 = 0, kc2 = 0;

    printf("\nInput image path ( *.bmp ): ");
    scanf("%s", &file_name);

    FLUSH;

    FILE *file;

    /* Open file */
    file = fopen(file_name, "r");
    if (file != NULL)
    {
        int pos = ftell(file);
        fseek(file, 0L, SEEK_END);
        int fileSize = ftell(file);

        fseek(file, pos, SEEK_SET);
        fclose(file);

        printf("File Found. SIZE [ %d ] \n", fileSize);          
        printf("Keycode 1: ");
        scanf("%d", &kc1);
        FLUSH;

        printf("Keycode 2: ");
        scanf("%d", &kc2);
        FLUSH;;

        ret = SaveGraphic(file_name, kc1, kc2);
        LOG_RET("SaveGraphic", ret);
    }
    else
        printf("File not found!\n");

}


void TestCleanGraphic(void)
{
    int ret = 0;
    int kc1 = 0, kc2 = 0;

    printf("Keycode 1: ");
    scanf("%d", &kc1);
    FLUSH;

    printf("Keycode 2: ");
    scanf("%d", &kc2);
    FLUSH;
    ret = CleanGraphic(kc1,kc2);
    LOG_RET("CleanGraphic", ret);
}


void TestCleanAllGraphics(void)
{
    int ret = CleanAllGraphics();
    LOG_RET("CleanAllGraphics", ret);
}

void TestPrintGraphic(void)
{
    int ret = 0;
    int kc1 = 0,kc2 = 0;

    printf("Keycode 1: ");
    scanf("%d", &kc1);
    FLUSH;

    printf("Keycode 2: ");
    scanf("%d", &kc2);
    FLUSH;

    ret = PrintGraphic(kc1, kc2);
    LOG_RET("PrintGraphic", ret);
}
