/**
 * 	Copyright (c) 2013 Perto S.A. All rights reserved.
 *
 * \file ptptrpro.h
 *
 * \brief PTPTRPRO API definitions.
 *
 * \author Abilio Faria
 */

#ifndef PTPTRPRO_H
#define PTPTRPRO_H


#ifdef __cplusplus
extern "C" {
#endif
/*!
 * PTPTRPROSHARED_EXPORT O.S. definition
 * This is required to use ptptrpro.h outside Qt.
 */

#ifdef _WIN32
#ifndef PTPTRPROSHARED_EXPORT
#define PTPTRPROSHARED_EXPORT   __declspec(dllexport)
#endif // PTPTRPROSHARED_EXPORT
#else // _WIN32
#ifndef BBX_TMF_PTR
    #ifndef PTPTRPROSHARED_EXPORT
        #define PTPTRPROSHARED_EXPORT   __attribute__((visibility("default")))
    #endif // PTPTRPROSHARED_EXPORT

#else
    #define PTPTRPROSHARED_EXPORT __attribute__((visibility("hidden")))
#endif

#endif // _WIN32

/*!
 *  Definitions
 */
/*! Return codes */
#define PTR_OK                   1   /*!< success */
#define PTR_ERROR               -1   /*!< error */
#define PTR_ERR_UNSUPPORTED     -2   /*!< not supported */
#define PTR_ERR_INVALIDPARAM    -3   /*!< invalid parameter */
#define PTR_ERR_NOTOPENED       -4   /*!< communication not opened */
#define PTR_ERR_DETACHED        -5   /*!< communication closed */
#define PTR_ERR_NOMEM           -6   /*!< memory not available */
#define PTR_ERR_NOTIMPL         -7   /*!< not implemented */
#define PTR_ERR_PROTNOTDEF      -8   /*!< protocol not defined */
#define PTR_ERR_USB_OPEN        -9   /*!< error opening USB device */
#define PTR_ERR_USB_CONFIG      -10  /*!< error setting USB configuration */
#define PTR_ERR_USB_CLAIM       -11  /*!< error claiming USB interface */
#define PTR_ERR_USB_RELEASE     -12  /*!< error releasing USB interface */
#define PTR_ERR_USB_CLOSE       -13  /*!< error closing USB handle */
#define PTR_ERR_NOPRESENCE      -14  /*!< error printer not presence */
#define PTR_ERR_PORTNOTFOUND    -15  /*!< error port not found */
#define PTR_ERR_IOERROR         -16  /*!< error I/O */
#define PTR_ERR_NORESOURCES		-17  /*!< error no resources */

/*! Protocol types */
#define PTR_PROT_NOTDEF      0          /*!< Protocol not defined */
#define PTR_PROT_PERTO       100        /*!< Perto protocol */
#define PTR_PROT_ESCPOS      101        /*!< ESC/POS protocol */

#define PTR_ID_BUF_LEN       16         /*!< size of the buffer (GetPrinterID) */

/*! piBaudRate values (SetBaudRate) */
#define PTR_BAUDRATE_9600       9600    /*!< 9600 bps */
#define PTR_BAUDRATE_19200      19200   /*!< 19200 bps */
#define PTR_BAUDRATE_115200     115200  /*!< 115200 bps */

/*! Font type (SetTextProperties) */
#define PTR_FONT_NORMAL         0       /*!< normal */
#define PTR_FONT_NARROW         1       /*!< narrow */
#define PTR_FONT_EXPANDED       2       /*!< expanded */

/*! Printer status bits (GetPrinterStatus) */
#define PTR_ST_OK                       0       /*!< operational */
#define PTR_ST_OFFLINE                  0x0001  /*!< off line */
#define PTR_ST_NOPAPER                  0x0002  /*!< paper out */
#define PTR_ST_LOWPAPER                 0x0004  /*!< paper low */
#define PTR_ST_HEADUP                   0x0008  /*!< head up */
#define PTR_ST_JAMMED                   0x0010  /*!< jammed */
#define PTR_ST_BUSY                     0x0020  /*!< busy */
#define PTR_ST_HEAD_TEMP_HIGH           0x0040  /*!< head temperature high */
#define PTR_ST_MOTOR_TEMP_HIGH          0x0080  /*!< motor temperature high */
#define PTR_ST_COVEROPEN                0x0100  /*!< cover openned */
#define PTR_ST_ONLINE                   0x0200  /*!< on line */
#define PTR_ST_UNKNOWN                  0x0400  /*!< unknown */
#define PTR_ST_VALIDATION_COVEROPEN     0x0800  /*!< unknown */

/*! Drawer identifier (GetDrawerStatus) */
#define PTR_DRAWERONE           1       /*!< drawer 1 */
#define PTR_DRAWERTWO           2       /*!< drawer 2 */

/*! Drawer status (GetDrawerStatus) */
#define PTR_DRAWER_ST_CLOSED    0       /*!< drawer closed */
#define PTR_DRAWER_ST_OPENED    1       /*!< drawer opened */
#define PTR_DRAWER_ST_UNKNOWN   0x0040  /*!< unknown */

/*! Bar code types (PrintBarCode) */
#define PTR_BAR_CODE_UPCA       65      /*!< UPCA */
#define PTR_BAR_CODE_UPCE       66      /*!< UPCE */
#define PTR_BAR_CODE_EAN13      67      /*!< EAN13 */
#define PTR_BAR_CODE_EAN8       68      /*!< EAN8 */
#define PTR_BAR_CODE_CODE39     69      /*!< CODE39 */
#define PTR_BAR_CODE_ITF        70      /*!< ITF */
#define PTR_BAR_CODE_CODABAR    71      /*!< CODABAR */
#define PTR_BAR_CODE_CODE93     72      /*!< CODE93 */
#define PTR_BAR_CODE_CODE128    73      /*!< CODE128 */
#define PTR_BAR_CODE_CODE32     90      /*!< CODE32 */
#define PTR_BAR_CODE_PDF417     128     /*!< PDF-417 */

/*! Bar code width (PrintBarCode) */
#define PTR_BAR_CODE_WIDTH_SLIM        0    /*!< slim width */
#define PTR_BAR_CODE_WIDTH_MEDIUM      1    /*!< medium width */
#define PTR_BAR_CODE_WIDTH_WIDE        2    /*!< wide width */

/*! Bar code text pos (PrintBarCode) */
#define PTR_BAR_CODE_POS_NO            0    /*!< not apply */
#define PTR_BAR_CODE_POS_ABOVE         1    /*!< positioned above */
#define PTR_BAR_CODE_POS_BELOW         2    /*!< positioned below */
#define PTR_BAR_CODE_POS_ABOVEANDBELOW 3    /*!< positioned above and below */

/*! Bar code text type (PrintBarCode) */
#define PTR_BAR_CODE_FONT_NORMAL  0     /*!< normal */
#define PTR_BAR_CODE_FONT_NARROW  1     /*!< narrow */

/*! Printer media status bits (GetMediaStatus) */
#define PTR_MEDIA_ST_ATEXIT                 0x0001   /*!< positioned at exit */
#define PTR_MEDIA_ST_LOWPAPER               0x0004   /*!< low paper */
#define PTR_MEDIA_ST_NOPAPER                0x0008   /*!< no paper */
#define PTR_MEDIA_ST_UNKNOWN                0x0040   /*!< unknown */
#define PTR_MEDIA_ST_VALIDATION_NOPAPER     0x0100   /*!< unknown */

/*! Dither mode (PrintBmpImage) */
#define PTR_IMAGE_DITHER_2X2    0   /*!< dither 2x2 */
#define PTR_IMAGE_DITHER_3X3    1   /*!< dither 3x3 */
#define PTR_IMAGE_DITHER_4X4    2   /*!< dither 4x4 */

/*! Paper cur type (MakeCut) */
#define PTR_CUT_FULL            0   /*!< full cut */
#define PTR_CUT_PARTIAL         1   /*!< partial cut */

/*! Justification (SetJustification) */
#define PTR_POS_LEFT            0   /*!< left justified */
#define PTR_POS_CENTER          1   /*!< center justified */
#define PTR_POS_RIGHT           2   /*!< right justified */

/*! Character code table (SetCharCodeTable) */
#define PTR_CODE_TABLE_PC437    0   /*!< USA, european standard */
#define PTR_CODE_TABLE_PC850    2   /*!< multilingual */
#define PTR_CODE_TABLE_PC860    3   /*!< portuguese */
#define PTR_CODE_TABLE_PC863    4   /*!< french-canadian (not supported) */
#define PTR_CODE_TABLE_PC865    5   /*!< nordic (not supported) */
#define PTR_CODE_TABLE_PC858    19  /*!< right justified */
#define PTR_CODE_TABLE_ABICOMP  27  /*!< Abcomp */

/*! Character size (SetCharSize) */
#define PTR_CHAR_SIZE_NORMAL        0x00    /*!< normal */
#define PTR_CHAR_SIZE_DOUBLE_HEIGHT 0x01    /*!< double-width and double-height */
#define PTR_CHAR_SIZE_DOUBLE_WIDTH  0x10    /*!< double-width and double-height */
#define PTR_CHAR_SIZE_DOUBLE        0x11    /*!< double-width and double-height */

/*! Type value (PRINTER_INFO) */
#define PTR_INFO_AUTOCUTTER     0x0001      /*!< automatic cutter */

/*! Printer mode (SetOperationMode) */
#define PTR_MODE_PRINTER		0
#define PTR_MODE_VALIDATION     1


/*! Printer QrCode Model (SetQrCodeModel) */
#define PTR_QRCODE_MODELONE         1       /*!< Model 1 (not supported) */
#define PTR_QRCODE_MODELTWO         2       /*!< Model 2 */


/*! Printer QrCode Error Correction (SetQrCodeErrorCorretion) */
#define PTR_QRCODE_LEVEL_L    0       /*!< Error Corretion Level L 7 */
#define PTR_QRCODE_LEVEL_M    1       /*!< Error Corretion Level M 15 */
#define PTR_QRCODE_LEVEL_Q    2       /*!< Error Corretion Level Q 25 */
#define PTR_QRCODE_LEVEL_H    3       /*!< Error Corretion Level H 30 */


/*! Printer QrCode Resolution (SetQrCodeResolution) */
#define PTR_QRCODE_FACTOR1          1       /*!< Factor 1 */
#define PTR_QRCODE_FACTOR2          2       /*!< Factor 2 */
#define PTR_QRCODE_FACTOR3          3       /*!< Factor 3 */
#define PTR_QRCODE_FACTOR4          4       /*!< Factor 4 */
#define PTR_QRCODE_FACTOR5          5       /*!< Factor 5 */
#define PTR_QRCODE_FACTOR6          6       /*!< Factor 6 */
#define PTR_QRCODE_FACTOR7          7       /*!< Factor 7 */


/*!
 *  Structures
 */

/*! printer information (GetPrinterInfo) */
typedef struct PrinterInfoStruct {
    int printer_model;      /*!< printer model */
    int type;               /*!< printer type */
    char rom_version[12];   /*!< printer version */
} PrinterInfoType;



/*!
 *  Prototypes (83 functions)
 */

/*!
 * Communication functions
 */
PTPTRPROSHARED_EXPORT int SetProtocol(int protocol);
PTPTRPROSHARED_EXPORT int OpenPort(const char *port, int baud_rate, char parity, int data_size, int stop_bit);
PTPTRPROSHARED_EXPORT int ClosePort(void);
PTPTRPROSHARED_EXPORT int SendData(const char *data, int size);
PTPTRPROSHARED_EXPORT int GetData(char *data, int size);

/*!
 * Operation mode function
 */
PTPTRPROSHARED_EXPORT int SetOperationMode(int mode);

/*!
 * Status functions
 */
PTPTRPROSHARED_EXPORT int Reset(void);
PTPTRPROSHARED_EXPORT int RealTimeRequest(int request);
PTPTRPROSHARED_EXPORT int GetPrinterStatus(int *status);
PTPTRPROSHARED_EXPORT int GetMediaStatus(int *status);
PTPTRPROSHARED_EXPORT int ptptrpro_GetMediaStatus(int *status);
PTPTRPROSHARED_EXPORT int GetDrawerStatus(int drawer_id, int *status);
PTPTRPROSHARED_EXPORT int GetPrinterInfo(PrinterInfoType *info);
PTPTRPROSHARED_EXPORT int GetRealTimeStatus(int type, int *status, int time);

/*!
 * Print functions
 */
PTPTRPROSHARED_EXPORT int PrintAndLineFeed(int size);
PTPTRPROSHARED_EXPORT int PrintAndPaperFeed(int size);
PTPTRPROSHARED_EXPORT int PrintInPageMode(void);
PTPTRPROSHARED_EXPORT int MakePrint(void);
PTPTRPROSHARED_EXPORT int GetPartialLineFeed(int lines);
PTPTRPROSHARED_EXPORT int PaperFeed(int size);
PTPTRPROSHARED_EXPORT int SetDumpEnable(void);

/*!
 * Paper cut functions
 */
PTPTRPROSHARED_EXPORT int MakeCut(int type);
PTPTRPROSHARED_EXPORT int MakeCutnLines(int type, int lines);

/*!
 * Bar code functions
 */
PTPTRPROSHARED_EXPORT int SetHriFont(int font);
PTPTRPROSHARED_EXPORT int SetHriPosition(int pos);
PTPTRPROSHARED_EXPORT int PrintBarCode(int type, int height, int width, int left_margin,
                                       int text_pos, int text_font, const char *data);

/*!
 * Bit image functions
 */
PTPTRPROSHARED_EXPORT int PrintBmpImage(const char *file_name, float height, float width,
                                        int degree, int left_margin, int dither_mode);
PTPTRPROSHARED_EXPORT int PrintBmp8(const char *file_name, float height, float width,
                                    int degree, int left_margin, int dither_mode);
PTPTRPROSHARED_EXPORT int PrintRasterBitImage(int mode, int width, int height, unsigned char *data);
PTPTRPROSHARED_EXPORT int PrintDownloadedBitImage(int mode);
PTPTRPROSHARED_EXPORT int SetDownloadedBitImage(int x, int y, unsigned char *data);
PTPTRPROSHARED_EXPORT int SetBitImageMode(int mode, unsigned char *data, int size);

PTPTRPROSHARED_EXPORT int SaveGraphic(const char * file_name, int kc1, int kc2);
PTPTRPROSHARED_EXPORT int PrintGraphic(int kc1, int kc2);
PTPTRPROSHARED_EXPORT int CleanGraphic(int kc1, int kc2);
PTPTRPROSHARED_EXPORT int CleanAllGraphics(void);
PTPTRPROSHARED_EXPORT int GetMemoryAvailabled(int *memory);

/*!
 * Character functions
 */
PTPTRPROSHARED_EXPORT int SetPrintMode(int mode);
PTPTRPROSHARED_EXPORT int SetCharCodeTable(int table);
PTPTRPROSHARED_EXPORT int SetTextProperties(int font, bool italic, bool underlined, bool bold);
PTPTRPROSHARED_EXPORT int SetFontType(int font);
PTPTRPROSHARED_EXPORT int SetCharSize(int size);
PTPTRPROSHARED_EXPORT int SetItalicEnable(void);
PTPTRPROSHARED_EXPORT int SetItalicDisable(void);
PTPTRPROSHARED_EXPORT int SetPrintModeUnderline(int mode);
PTPTRPROSHARED_EXPORT int SetBoldEnable(int mode);
PTPTRPROSHARED_EXPORT int SetBoldDisable(void);
PTPTRPROSHARED_EXPORT int SetDoubleStrike(int mode);
PTPTRPROSHARED_EXPORT int SetReverseBlackWhite(int mode);
PTPTRPROSHARED_EXPORT int SetCondensedEnable(void);
PTPTRPROSHARED_EXPORT int SetCondensedDisable(void);
PTPTRPROSHARED_EXPORT int SetExpandedModeEnable(void);
PTPTRPROSHARED_EXPORT int SetExpandedModeDisable(void);
PTPTRPROSHARED_EXPORT int SetInternationalCharSet(int type);
PTPTRPROSHARED_EXPORT int SetUserChar(int mode);
PTPTRPROSHARED_EXPORT int DefineUserChar(int height, int start_code, int final_code, char *data);
PTPTRPROSHARED_EXPORT int CancelUserChar(int code);

/*!
 * Layout functions
 */
PTPTRPROSHARED_EXPORT int SetMotionUnits(int x, int y);
PTPTRPROSHARED_EXPORT int SetPrintingArea(int x, int y, int width, int height);
PTPTRPROSHARED_EXPORT int SetPrintingAreaHeight(int height);
PTPTRPROSHARED_EXPORT int SetPageLength(int lines);
PTPTRPROSHARED_EXPORT int SetLeftMargin(int size);
PTPTRPROSHARED_EXPORT int SetRightMargin(int size);
PTPTRPROSHARED_EXPORT int SetJustification(int pos);
PTPTRPROSHARED_EXPORT int SetCharSpacing(int size);
PTPTRPROSHARED_EXPORT int SetLineSpacing(int size);
PTPTRPROSHARED_EXPORT int SetDefaultLineSpacing(void);
PTPTRPROSHARED_EXPORT int SetRelativePrintPosition(int pos);
PTPTRPROSHARED_EXPORT int SetAbsolutePrintPosition(int pos);
PTPTRPROSHARED_EXPORT int SetAbsoluteVerticalPrintPosition(int pos);
PTPTRPROSHARED_EXPORT int SetHorizTabPosition(unsigned char *data);
PTPTRPROSHARED_EXPORT int SetCharacterPerLine(int mode);
PTPTRPROSHARED_EXPORT int SetAutomaticLineFeed(int mode);
PTPTRPROSHARED_EXPORT int SetUpsideDownMode(int mode);
PTPTRPROSHARED_EXPORT int SetPrintDensity(int mode);
PTPTRPROSHARED_EXPORT int SetMode48Columns(void);
PTPTRPROSHARED_EXPORT int SetRotation(int mode);


/*!
 * QrCode functions
 */

PTPTRPROSHARED_EXPORT int SetQrCodeModel(int model);
PTPTRPROSHARED_EXPORT int SetQrCodeResolution(int factor);
PTPTRPROSHARED_EXPORT int SetQrCodeErrorCorretion(int level);
PTPTRPROSHARED_EXPORT int StoreQrCodeData(char *data, int size);
PTPTRPROSHARED_EXPORT int MakeQrCode(void);


/*!
 * Cancel functions
 */
PTPTRPROSHARED_EXPORT int CancelLastCharacterSent(void);
PTPTRPROSHARED_EXPORT int CancelLastLineSended(void);

/*!
 * Print mode functions
 */
PTPTRPROSHARED_EXPORT int SetStandardMode(void);
PTPTRPROSHARED_EXPORT int SetPageMode(void);

/*!
 * Mechanism control functions
 */
PTPTRPROSHARED_EXPORT int GeneratePulse(int pin, int time_on, int time_off);
PTPTRPROSHARED_EXPORT int SetPeripheralDevice(int status);
PTPTRPROSHARED_EXPORT int SetKeyboard(int mode);
PTPTRPROSHARED_EXPORT int SetDrawerOn(int mode);

/*!
 * Macro functions
 */
PTPTRPROSHARED_EXPORT int StartEndMacroDefinition(void);
PTPTRPROSHARED_EXPORT int ExecuteMacro(int times, int waiting_time, int mode);

/*!
 * Log functions
 */
PTPTRPROSHARED_EXPORT int EnableLog(bool mode, const char *file_name);

#ifdef __cplusplus
}
#endif

#endif // PTPTRPRO_H
